#!/usr/bin/python

##   Quack
##   Copyright (C) 2007 Mads Villadsen <maxximum@krakoa.dk>
    
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
   
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
   
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from xmlrpclib import ServerProxy
import urllib2

try:
    import json # python-json
except:
    print "python-json not found"
    sys.exit(1)

class Jaiku:
    __user_information_url = ".jaiku.com/json"
    __user_presence_url = ".jaiku.com/presence/json"
    __user_stream_url = ".jaiku.com/feed/json"
    __xmlrpc_endpoint = "http://api.jaiku.com/xmlrpc"

    __icons = {
        "car": 301,
        "alarm clock": 302,
        "loudspeaker": 303,
        "tram": 304,
        "casette": 305,
        "underwear": 306,
        "rollerblade": 307,
        "uzi": 308,
        "scoop": 309,
        "bomb": 310,
        "bra": 311,
        "videotape": 312,
        "cigarettes": 313,
        "vinyl": 314,
        "champaign": 315,
        "airport": 316,
        "bus": 317,
        "grumpy": 318,
        "coffee": 319,
        "camera": 320,
        "basketball": 321,
        "beer": 322,
        "binoculars": 323,
        "boiler": 324,
        "walk": 325,
        "wallclock": 326,
        "trashcan": 327,
        "tv": 328,
        "computer": 329,
        "videocamera": 330,
        "game": 331,
        "cone": 332,
        "driller": 333,
        "popcorn": 334,
        "play": 335,
        "disc": 336,
        "event": 337,
        "exclamationmark": 338,
        "football": 339,
        "": 340,
        "fork": 341,
        "gameboy": 342,
        "grenade": 343,
        "hand": 344,
        "hanger": 345,
        "ear muffs": 346,
        "love": 347,
        "balloons": 348,
        "clock": 349,
        "barrier": 350,
        "": 351,
        "megaphone": 352,
        "microwave": 353,
        "book": 354,
        "middle finger": 355,
        "notes": 356,
        "question": 357,
        "rollator": 358,
        "shuttlecock": 359,
        "salt": 360,
        "scull": 361,
        "sk8": 362,
        "leep": 363,
        "snorkeling": 364,
        "snowflake": 365,
        "soda": 366,
        "song": 367,
        "spray": 368,
        "sticks": 369,
        "storm": 370,
        "straitjacket": 371,
        "metro": 372,
        "luggage": 373,
        "sun": 374,
        "taxi": 375,
        "technics": 376,
        "toaster": 377,
        "train": 378,
        "wheelchair": 379,
        "zippo": 380,
        "ice cream": 381,
        "movie": 382,
        "makeup": 383,
        "bandaid": 384,
        "wine": 385,
        "clean": 386,
        "blading": 387,
        "bike": 388,
        "pils": 389,
        "picnic": 390,
        "lifejacket": 391,
        "home": 392,
        "happy": 393,
        "toiletpaper": 394,
        "theatre": 395,
        "shop": 396,
        "search": 397,
        "cloudy": 398,
        "Hurry": 399,
        "Morning": 400,
        "": 401,
        "Itsaboy": 402
        }

    def __init__(self, uname, pkey):
        self.username = uname
        self.personal_key = pkey
        self.location = ""
        self.jaiku = ServerProxy(Jaiku.__xmlrpc_endpoint)
        self.contacts_cache = {}

        self.location = self.__jaiku_get_location()

    def __get_icon_id(self, icon_name):
        retval = 300
        try:
            retval = Jaiku.__icons[icon_name]
        except KeyError:
            # just use the default value then
            pass

        return retval

    def __jaiku_get_location(self):
        retval = ""

        up = self.get_user_presence(self.username)

        if (up != None):
            location = up["location"]
            neighbourhood = location["neighbourhood"]
            city = location["city"]
            country = location["country"]

            if (neighbourhood != ""):
                retval += neighbourhood
            if (city != ""):
                if (retval != ""):
                    retval += ", "
                retval += city
            if (country != ""):
                if (retval != ""):
                    retval += ", "
                retval += country

        return retval

    def __jaiku_send(self, message, icon, location, generated):
        retval = False

        try:
            ret = self.jaiku.presence.send({"user": self.username,
                                            "personal_key": self.personal_key,
                                            "message": message,
                                            "icon": icon,
                                            "location": location,
                                            "generated": generated
                                            })
            if (ret["status"] == "ok"):
                retval = True
        except:
            pass

        return retval

    def get_location(self):
        return self.location

    def set_location(self, location):
        self.location = location
        return self.__jaiku_send("", "", self.location, True)

    def send_message(self, message, icon):
        icon_id = self.__get_icon_id(icon)
        return self.__jaiku_send(message, icon_id, self.location, False)

    def get_user_information(self, username):
        try:
            uf = urllib2.urlopen("http://" + username + Jaiku.__user_information_url + "?user=" + self.username + "&personal_key=" + self.personal_key)
            output = uf.read()
            uf.close()

            return json.read(output)
            
        except urllib2.HTTPError, (code):
            print code
            return None

        except urllib2.URLError, (code):
            print code
            return None

    def get_user_presence(self, username):
        try:
            uf = urllib2.urlopen("http://" + username + Jaiku.__user_presence_url + "?user=" + self.username + "&personal_key=" + self.personal_key)
            output = uf.read()
            uf.close()

            return json.read(output)
            
        except urllib2.HTTPError, (code):
            print code
            return None
        except urllib2.URLError, (code):
            print code
            return None

    def get_user_stream(self, username):
        try:
            uf = urllib2.urlopen("http://" + username + Jaiku.__user_stream_url + "?user=" + self.username + "&personal_key=" + self.personal_key)
            output = uf.read()
            uf.close()

            o = json.read(output)
            return o
            
        except urllib2.HTTPError, (code):
            print code
            return None
        except urllib2.URLError, (code):
            print code
            return None
        except json.ReadException:
            print "Error reading JSON for user: " + username
            return None

    def get_contacts(self):
        user = self.get_user_information(self.username)
        if (user != None):
            return user["contacts"]
        else:
            return None

    def get_new_contact_info(self):
        # print "getting updates"
        # TODO: rename method to something more appropriate - get_new_jaikus_from_contacts ?
        new_updates = []

        user = self.get_user_information(self.username)
        if (user != None):
            for u in user["contacts"]:
                nick = u["nick"]
                current_user_all_jaikus = self.get_user_stream(nick)
                # TODO: use a nice pythonic object instead of the JSON one
                if (current_user_all_jaikus != None):
                    if (not self.contacts_cache.has_key(nick)):
                        # print "first time check (" + nick + ") - only grab the latest jaiku"
                        latest_jaiku = current_user_all_jaikus["stream"][0]
                        latest_jaiku_timestamp = latest_jaiku["created_at"]
                        self.contacts_cache[nick] = latest_jaiku_timestamp
                        new_updates.append(latest_jaiku)
                    else:
                        # print "not first time check (" + nick + ") - grab everything later than: " + self.contacts_cache[nick]
                        for j in current_user_all_jaikus["stream"]:
                            if (j["created_at"] > self.contacts_cache[nick]):
                                new_updates.append(j)
                        # update cache with the latest timestamp
                        latest_jaiku = current_user_all_jaikus["stream"][0]
                        latest_jaiku_timestamp = latest_jaiku["created_at"]
                        self.contacts_cache[nick] = latest_jaiku_timestamp

        return new_updates


