#!/usr/bin/python

##   Quack
##   Copyright (C) 2007 Mads Villadsen <maxximum@krakoa.dk>
    
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
   
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
   
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from threading import Thread, Timer
import urllib2
import os, time

try:
    import pynotify # notify-python
except:
    print "pynotify not found"
    sys.exit(1)

try:
    import pygtk
    pygtk.require ("2.0")
    import gtk, gtk.glade, gnome, gnome.ui, gobject, gconf
except:
    print "pyGTK 2.0 not found."
    sys.exit(1)

from globals import *
import jaiku

class JaikuApp:
   def __init__(self, is_applet):
       self.quitting = False # used for exiting threads
       self.force_update = True # force an update on startup
       self.__hidden = False
       self.__is_applet = is_applet

       self.wTree = None
       try:
           self.wTree = gtk.glade.XML (gladefile, "quack_app")
       except:
           self.wTree = gtk.glade.XML (devel_gladefile, "quack_app")
       self.main_window = self.wTree.get_widget ("quack_app")

       if (self.__is_applet):
           self.hide()

       self.location_entry = self.wTree.get_widget("location_entry")
       self.jaiku_entry = self.wTree.get_widget("jaiku_entry")

       self.contacts_iconview = self.wTree.get_widget("contacts_iconview")


       try:
           self.main_window.set_icon_from_file(window_icon_file)
           gtk.window_set_default_icon_from_file(window_icon_file)
       except:
           pass


       self.conf_client = gconf.client_get_default()

       dic = {
           "on_quack_app_destroy": self.destroy,
           "on_quit_activate": self.destroy,
           "on_about_activate": self.about_activate,
           "on_preferences_activate": self.preferences_activate, 
           "on_jaiku_entry_activate": self.on_submit_button_clicked,
           "on_location_entry_activate": self.on_submit_button_clicked,
           "on_post_button_clicked": self.on_submit_button_clicked
           }
       self.wTree.signal_autoconnect(dic)



       # settings - should be exposed in a configuration GUI
       self.jaiku_check_interval = 90 # seconds
       self.show_updates_for_jaiku_user = self.conf_client.get_bool("/apps/" + name + "/show_updates_for_jaiku_user") # should we show updates from the "Jaiku" user
       if (self.show_updates_for_jaiku_user == None):
           self.show_updates_for_jaiku_user = False
       self.jaiku_username = self.conf_client.get_string("/apps/" + name + "/username")
       if (self.jaiku_username == None):
           self.jaiku_username = ""
       self.jaiku_user_key = self.conf_client.get_string("/apps/" + name + "/user_key")
       if (self.jaiku_user_key == None):
           self.jaiku_user_key = ""


       self.jaiku_instance = None
       if (self.jaiku_username != "" and self.jaiku_user_key != ""):
           self.jaiku_instance = jaiku.Jaiku(self.jaiku_username, self.jaiku_user_key)
       else:
           user_not_set_dialogwTree = None
           try:
               user_not_set_dialogwTree = gtk.glade.XML(gladefile, "user_not_set_dialog")
           except:
               user_not_set_dialogwTree = gtk.glade.XML(devel_gladefile, "user_not_set_dialog")
           user_not_set_dialog = user_not_set_dialogwTree.get_widget("user_not_set_dialog")
           user_not_set_dialog.set_transient_for(self.main_window)
           result = user_not_set_dialog.run()
           user_not_set_dialog.destroy()
           self.preferences_activate(self.main_window)
           

       self.jaiku_checker_thread = Thread(target=self.jaiku_checker)
       self.jaiku_checker_thread.start()

       if (self.jaiku_instance != None):
           self.location_entry.set_text(self.jaiku_instance.get_location())

           # ensure that we have all avatar icons
           contacts = self.jaiku_instance.get_contacts()
           self.get_and_store_contacts_icons(contacts)

           contacts_model = gtk.ListStore(str, gtk.gdk.Pixbuf)
           self.contacts_iconview.set_model(contacts_model)
           self.contacts_iconview.set_text_column(0)
           self.contacts_iconview.set_pixbuf_column(1)

           for c in contacts:
               tmp_icon_name = basedir + c["nick"] + ".png"
               icon_pixbuf = gtk.gdk.pixbuf_new_from_file(tmp_icon_name)
               icon_name = c["nick"]
               contacts_model.append([icon_name, icon_pixbuf])


   def destroy(self, widget, data=None):
       #print "destroy"
       self.quitting = True
       gtk.main_quit()

   def shutdown_threads(self):
       self.quitting = True

   def hide(self):
       self.main_window.hide()
       self.__hidden = True

   def show(self):
       self.main_window.show()
       self.__hidden = False

   def is_hidden(self):
       return self.__hidden

   def about_activate(self, widget):
       about_dialogwTree = None
       try:
           about_dialogwTree = gtk.glade.XML(gladefile, "about")
       except:
           about_dialogwTree = gtk.glade.XML(devel_gladefile, "about")

       about_dialog = about_dialogwTree.get_widget("about")

       about_dialog.set_transient_for(self.main_window)
       
       about_dialog.set_property('name', program_name)
       about_dialog.set_property('version', program_version)
       
       result = about_dialog.run()
       about_dialog.destroy()

   def preferences_activate(self, widget):
       preferences_dialogwTree = None
       try:
           preferences_dialogwTree = gtk.glade.XML(gladefile, "preferences")
       except:
           preferences_dialogwTree = gtk.glade.XML(devel_gladefile, "preferences")

       preferences_dialog = preferences_dialogwTree.get_widget("preferences")
       username_entry = preferences_dialogwTree.get_widget("username_entry")
       user_key_entry = preferences_dialogwTree.get_widget("user_key_entry")
       show_messages_from_jaiku_checkbutton = preferences_dialogwTree.get_widget("show_messages_from_jaiku_checkbutton")
       username_entry.set_text(self.jaiku_username)
       user_key_entry.set_text(self.jaiku_user_key)
       show_messages_from_jaiku_checkbutton.set_active(self.show_updates_for_jaiku_user)

       preferences_dialog.set_transient_for(self.main_window)
       
       result = preferences_dialog.run()

       if (result == gtk.RESPONSE_CLOSE):
           new_username = username_entry.get_text()
           new_user_key = user_key_entry.get_text()
           new_show_messages_from_jaiku = show_messages_from_jaiku_checkbutton.get_active()

           self.conf_client.set_string("/apps/" + name + "/username", new_username)
           self.conf_client.set_string("/apps/" + name + "/user_key", new_user_key)
           self.conf_client.set_bool("/apps/" + name + "/show_updates_for_jaiku_user", new_show_messages_from_jaiku)

           # TODO: these should be set using gconf callbacks
           self.jaiku_username = new_username
           self.jaiku_user_key = new_user_key
           self.show_updates_for_jaiku_user = new_show_messages_from_jaiku

           if (self.jaiku_username != "" and self.jaiku_user_key != ""):
               self.jaiku_instance = jaiku.Jaiku(self.jaiku_username, self.jaiku_user_key)

       preferences_dialog.destroy()

   def on_submit_button_clicked(self, button):
       #print "posting..."

       location = self.location_entry.get_text() # get string from location entry
       jaiku = self.jaiku_entry.get_text() # get string from jaiku entry
       success = False
       if (jaiku == ""):
           success = self.send_location(location)
       else:
           success = self.send_jaiku(jaiku, location)
       
       if (success):
           self.__update_status_with_pop("Jaiku sent")
       else:
           self.__update_status_with_pop("Error sending jaiku")

   def __update_status_with_pop(self, s):
       self.wTree.get_widget("appbar").push(s)
       t = Timer(3.0, self.wTree.get_widget("appbar").pop) # pop this after 3 seconds
       t.start()

   def send_location(self, location):
       if (self.jaiku_instance != None):
           return self.jaiku_instance.set_location(location)
       else:
           return False

   def send_jaiku(self, jaiku, location):
       if (self.jaiku_instance != None):
           return (self.jaiku_instance.set_location(location) and self.jaiku_instance.send_message(jaiku, 300))
       else:
           return False

   def jaiku_checker(self):
       while True:
           for i in range(self.jaiku_check_interval):
               time.sleep(1)
               if (self.quitting or self.force_update):
                   #print "breaking for-loop"
                   self.force_update = False
                   break
           if (self.quitting):
               #print "breaking while-loop"
               break

           # check for updates
           # print "checking for updates"
           updates = self.jaiku_instance.get_new_contact_info()
           if (len(updates) > 0):
               gobject.idle_add(self.show_updates, updates)

   def show_updates(self, updates):
       for u in updates:
           n_title = u["user"]["first_name"] + " " + u["user"]["last_name"] + " (" + u["user"]["nick"] + ")"
           n_content = u["title"] + "\n\n" + u["created_at_relative"] # + "\n\n" + u["url"]
           n_icon = basedir + u["user"]["nick"] + ".png"

           if (u["user"]["nick"] == "jaiku" and not self.show_updates_for_jaiku_user):
               pass
           else:
               n = pynotify.Notification(n_title, n_content, n_icon)
               n.set_urgency(pynotify.URGENCY_LOW)
               n.set_timeout(15*1000) # 15 seconds
               n.show()

   def get_and_store_contacts_icons(self, contacts):
       for c in contacts:
           tmp_icon_name = basedir + c["nick"] + ".png"
           #print "checking avatar icon: " + tmp_icon_name
           if (not(os.path.exists(tmp_icon_name))):
               #print "getting avatar icon: " + tmp_icon_name 
               uf = urllib2.urlopen(c["avatar"])
               output = uf.read()
               uf.close()
               f = open(tmp_icon_name, "w")
               f.write(output)
               f.close()

       
